%{
#include <stdio.h>

#define YYDEBUG 1

#define YYSTYPE int

/* yylex function, returns token type */
int yylex (void);

/* error function, called when error is detected */
void yyerror (char const *);

int store[256];  /* variable store, mostly unused */
%}

%token NUM VAR

%right '='
%left '+' '-'
%left '*' '/'
%right '!'


%%
stmts : exp ';' stmts  { printf("===== %d =====\n", $1); }
      | { }
      ;

exp		: VAR '=' exp { $$ = $3; store[$1] = $3; }
		| exp '+' exp { $$ = $1 + $3; }
		| exp '-' exp { $$ = $1 - $3; }
		| exp '*' exp { $$ = $1 * $3; }
		| exp '/' exp { $$ = $1 / $3; }
		| '!' exp { $$ = -$1; }  /* make '!' behave like unary minus */
		| VAR { $$ = store[$1]; }
                | NUM { $$ = $1; }
                ;
   
%%

int main (void)
{
    int c;
    
    for (c=0; c<256; c++)
        store[c] = 1;
        
    yydebug = YYDEBUG; /* for debugging */

    return yyparse ();
}

/* Called by yyparse on error.  */
void yyerror (char const *s)
{
     fprintf (stderr, "%s\n", s);
}

int yylex (void)
{
    int c = getchar();
    if (c>='0' && c<='9') {
        yylval = c-'0';
        return NUM;
    }
    else if (c>='a' && c<='d') {
        yylval = c;
        return VAR;
    }
        
    return c;
}
